#!/bin/bash

SCRIPT=`readlink -f $0`
SCRIPTPATH=`dirname $SCRIPT`

. $SCRIPTPATH/inc.sh

echo SCRIPTPATH=$SCRIPTPATH
echo BUILDPATH=$BUILDPATH

BUILDLOG=$SCRIPTPATH/build.log
TMPLOG=$SCRIPTPATH/tmp.log

#print first message to log file
echo "building started `date "+%Y-%m-%d %H:%M:%S"`" > $BUILDLOG

mkdir -p $BUILDPATH

cd $SCRIPTPATH

function msg {
	M="`date "+%Y-%m-%d %H:%M:%S"`: $1"
	echo $M
	echo $M >> $BUILDLOG
}

DO_INSTALL_PACKAGES="no"
DO_CLEAR="no"
DO_MAKE_CLEAN="no"
DO_PUBLISH="no"
DO_IRC="no"
DO_CHECK_IF_UPDATED="yes"
DO_USE_STDOUT_STDERR="no"

for i in $*
do
	case $i in
#    	--prefix=*)
#		PREFIX=`echo $i | sed 's/[-a-zA-Z0-9]*=//'`
#		;;
#   	--searchpath=*)
#		SEARCHPATH=`echo $i | sed 's/[-a-zA-Z0-9]*=//'`
#		;;
#    	--lib=*)
#		DIR=`echo $i | sed 's/[-a-zA-Z0-9]*=//'`
#		;;
    	--install-packages)
		DO_INSTALL_PACKAGES="yes"
		;;
    	--clear)
		DO_CLEAR="yes"
		;;
    	--make-clean)
		DO_MAKE_CLEAN="yes"
		;;
    	--publish)
		DO_PUBLISH="yes"
		;;
    	--irc)
		DO_IRC="yes"
		;;
    	--dont-check-if-updated)
		DO_CHECK_IF_UPDATED="no"
		;;
    	--use-stdout-stderr)
		DO_USE_STDOUT_STDERR="yes"
		;;
    	*)
		echo "$i: unknown option"
		echo "usage: ./make.sh <options>"
		echo "possible options are: --install-packages --clear --make-clean"
		echo "                      --publish --dont-check-if-updated --irc"
		echo "                      --use-stdout-stderr"
		exit 1
		;;
  	esac
done

BUILD="./makecad.sh ./makecnc.sh ./makearea.sh ./makeactp.sh ./makestartcad.sh"

if [ $DO_INSTALL_PACKAGES = "yes" ] ; then
	BUILD="./installpackages.sh $BUILD"
fi
if [ $DO_CLEAR = "yes" ] ; then
	BUILD="./clear.sh $BUILD"
fi
if [ $DO_PUBLISH = "yes" ] ; then
	BUILD="$BUILD ./copy.sh"
fi

if [ -a $BUILDPATH/heekscad ]
then
	msg "updating heekscad"
	cd $BUILDPATH/heekscad
	OLDCADREV=`svn info |grep Revision |cut -f2 -d' '`
	svn update
	CADREV=`svn info |grep Revision |cut -f2 -d' '`
else
	OLDCADREV="none"
	CADREV="none_new"
fi

if [ -a $BUILDPATH/heekscad/heekscnc ]
then
	msg "updating heekscnc"
	cd $BUILDPATH/heekscad/heekscnc
	OLDCNCREV=`svn info |grep Revision |cut -f2 -d' '`
	svn update
	CNCREV=`svn info |grep Revision |cut -f2 -d' '`
else
	OLDCNCREV="none"
	CNCREV="none_new"
fi

msg "OLDCADREV=$OLDCADREV"
msg "OLDCNCREV=$OLDCNCREV"
msg "CADREV=$CADREV"
msg "CNCREV=$CNCREV"

if [[ $CADREV = $OLDCADREV && $CNCREV = $OLDCNCREV ]] ; then
	if [ "$DO_CHECK_IF_UPDATED" = "yes" ] ; then
		msg "Versions have not changed. Not building. use --dont-check-if-updated"
		exit 0
	else
		msg "Same versions as before, but building."
	fi
fi

builds="yes"

cd $SCRIPTPATH
for i in $BUILD; do
	msg "running $i"
	if [ "$DO_USE_STDOUT_STDERR" = "yes" ] ; then
		DO_MAKE_CLEAN=$DO_MAKE_CLEAN $i
	else
		DO_MAKE_CLEAN=$DO_MAKE_CLEAN $i &> $TMPLOG
	fi
	if [ $? != "0" ]; then
		cat $TMPLOG >> $BUILDLOG
		cat $TMPLOG
		msg "$i failed"
		builds="no"
		break
	else
		cat $TMPLOG >> $BUILDLOG
		msg "$i ready"
	fi
done

DATE=`date "+%Y-%m-%d_%H-%M-%S"`

msg "DATE=$DATE"

cp -f $BUILDLOG $COPYPATH/

NAME="heekscad_runinplace_cad${CADREV}_cnc${CNCREV}_${DATE}"
BOTFILE="$SCRIPTPATH/botnews.txt"
BROKENFILE="$SCRIPTPATH/broken.txt"
LASTWORKINGFILE="$SCRIPTPATH/lastworking.txt"

msg "NAME=$NAME"

if [ $builds = "yes" ] ; then
	#works
	msg "build ready"
	if [ $DO_PUBLISH = "yes" ] ; then
		mkdir -p $OUTPATH
		cp -f $BUILDLOG "$OUTPATH/${NAME}_build_WORKS.log"
		OUTNAME="$OUTPATH/${NAME}.tar.bz2"
		cd $COPYPATH/..
		msg "making tarball"
		tar cjf $OUTNAME `basename "$COPYPATH"`
	fi
	if [ $DO_IRC = "yes" ] ; then
		if [ -f $BROKENFILE ] ; then
			echo "Build result: cad$CADREV with cnc$CNCREV: now works." > $BOTFILE
			rm $BROKENFILE
			echo "cad$CADREV with cnc$CNCREV" > $LASTWORKINGFILE
		fi
		#echo "Build result: cad$CADREV with cnc$CNCREV: works." > $BOTFILE
	fi
else
	#failed
	msg "build failed"
	if [[ $DO_PUBLISH = "yes" || $DO_IRC = "yes" ]] ; then
		mkdir -p $OUTPATH
		cp -f $BUILDLOG "$OUTPATH/${NAME}_build_DOES_NOT_WORK.log"
	fi
	if [ $DO_IRC = "yes" ] ; then
		echo "Build result: cad$CADREV with cnc$CNCREV: does not work. `if [ -f $LASTWORKINGFILE ];then echo \"Last working was: \`cat $LASTWORKINGFILE\`. \";fi`See http://celer.oni.biz/~celeron55/automation/heeks/${NAME}_build_DOES_NOT_WORK.log" > $BOTFILE
		touch $BROKENFILE
	fi
fi

exit 0

